@@ -32,7 +32,7 @@
 #include "dvdspanky.h"
 #include "config.h"
 
-static char gp_title[] = "dvdspanky 0.10.0 - (C) 2007 Jeffrey Grembecki\n\n";
+static char gp_title[] = "dvdspanky 0.10.1 - (C) 2007 Jeffrey Grembecki\n\n";
 static char gp_licence[] =
 "This program is free software; you can redistribute it and/or modify\n"
 "it under the terms of the GNU General Public License as published by\n"
@@ -50,36 +50,38 @@ static char gp_licence[] =
 "MA 02110-1301, USA.\n";
 
 static char gp_help[] =
-"%s -i file -o file [-bBpnNm12FILhv] [-s size] [-r rate] [-R rate] [-c channels] [-V pixels] [-H pixels] [-a mode] [-f filters]\n"
+"%s -i file [ -o file ] [-12bBFhILnOmNpTv] [-s size] [-r rate] [-R rate] [-c channels] [-V pixels] [-H pixels] [-a mode] [-f filters]\n"
 "\n"
-"Following; [ ] = default, { } = requirements, * = experimental\n"
+"following; [ ] = default, { } = requirements, * = experimental\n"
 "\n"
-"-i, --input=FILE       input file\n"
-"-o, --output=FILE      output filename prefix (w/o extension)\n"
-"-s, --size=KB          final mpg size in kilobytes\n"
-"-r, --vrate=RATE       video bitrate [auto] {600 - 7500}\n"
-"-R, --arate=RATE       audio bitrate [auto] {64 - 448, 192 is good}\n"
-"-c, --channels=CHANS   number of audio channels [auto] {1 - 5}\n"
-"-V, --vcrop=PIXELS     pos for crop, neg for black border [off]\n"
-"-H, --hcrop=PIXELS     as above [off]\n"
-"-b, --border           automatic black border [off]\n"
-"-B, --clip             clips edges to fit aspect [off]\n"
-"-P, --preview          previews 5 frames using feh then exits [off]\n"
-"-p, --pal              set pal [auto]\n"
-"-n, --ntsc             set ntsc [auto]\n"
-"-N, --ntscfix          fix variable framerate ntsc video [off] *\n"
-"-a, --aspect=MODE      aspect ratio 2=4:3, 3=16:9 [auto] {2,3}\n"
-"-m, --multi            2 pass mode [off]\n"
 "-1, --firstpass        1st pass only [off]\n"
 "-2, --secondpass       2nd pass only including multiplex [off]\n"
-"-M, --mplex            multiplex only [off]\n"
+"-a, --aspect=MODE      aspect ratio 2=4:3, 3=16:9 [auto] {2,3}\n"
+"-b, --border           automatic black border [off]\n"
+"-B, --clip             clips edges to fit aspect [off]\n"
+"-c, --channels=CHANS   number of audio channels [auto] {1 - 5}\n"
+"-f, --filters=FILTERS  specify aditional transcode filters [none]\n"
 "-F, --frameadjust      interlace video for framerate conversion [off] *\n"
-"-f, --filters=FILTERS  eg. -f pp=de,hqdn3d [off]\n"
-"-I, --nonice           do not run at nice priority 19 [off]\n"
-"-v, --verbose          be verbose [off]\n"
-"-L, --licence          show the program licence\n"
 "-h, --help             this help text\n"
-"\nSee man 1 dvdspanky for more information\n";
+"-H, --hcrop=PIXELS     as above [off]\n"
+"-i, --input=FILE       input file\n"
+"-I, --nonice           do not run at nice priority 19 [off]\n"
+"-L, --licence          show the program licence\n"
+"-m, --multi            2 pass mode [off]\n"
+"-M, --mplex            multiplex only [off]\n"
+"-n, --ntsc             set ntsc [auto]\n"
+"-N, --ntscfix          fix variable framerate ntsc video [off] *\n"
+"-o, --output=FILE      output filename prefix (w/o extension)\n"
+"-O, --normalise        normalise audio volume [off]\n"
+"-r, --vrate=RATE       video bitrate [auto] {600 - 7500}\n"
+"-p, --pal              set pal [auto]\n"
+"-P, --preview          previews 5 frames using feh then exits [off]\n"
+"-R, --arate=RATE       audio bitrate [auto] {64 - 448, 192 is good}\n"
+"-s, --size=KB          final mpg size in kilobytes\n"
+"-T, --postprocess      enable post processing filter [off]\n"
+"-v, --verbose          be verbose [off]\n"
+"-V, --vcrop=PIXELS     pos for crop, neg for black border [off]\n"
+"\nsee man 1 dvdspanky for more information\n";
 
 static struct option gp_longopts[] =
 {
@@ -98,25 +100,62 @@ static struct option gp_longopts[] =
 	{"licence", 0, 0, 'L'},
 	{"mplex", 0, 0, 'M'},
 	{"multi", 0, 0, 'm'},
+	{"multipass", 0, 0, 'm'},
 	{"ntsc", 0, 0, 'n' },
 	{"ntscfix", 0, 0, 'N'},
 	{"output", 1, 0, 'o'},
+	{"normalise", 0, 0, 'O'},
+	{"normalize", 0, 0, 'O'},
 	{"pal", 0, 0, 'p' },
 	{"preview", 0, 0, 'P'},
 	{"vrate", 1, 0, 'r'},
 	{"arate", 1, 0, 'R'},
 	{"size", 1, 0, 's'},
+	{"postprocess", 0, 0, 'T'},
 	{"vcrop", 1, 0, 'V'},
 	{"verbose", 0, 0, 'v'},
 };
 
-static char gp_shortopts[] = "12a:bBc:f:FhH:i:ILmnNo:pPr:R:s:vV:";
+static char gp_shortopts[] = "12a:bBc:f:FhH:i:ILmnNo:OpPr:R:s:TvV:";
 
 pcre *gp_idregex = NULL;
 pcre *gp_transregex = NULL;
 pcre *gp_tcvideoregex = NULL;
 pcre *gp_tcaudioregex = NULL;
 
+static void printerror(const char *p_format, ...)
+{
+	va_list argp;
+
+	fprintf(stderr, "[dvdspanky] error: ");
+	va_start(argp, p_format);
+	vfprintf(stderr, p_format, argp);
+	va_end(argp);
+	fprintf(stderr, "\n");
+}
+
+static void printwarning(const char *p_format, ...)
+{
+	va_list argp;
+
+	fprintf(stderr, "[dvdspanky] warning: ");
+	va_start(argp, p_format);
+	vfprintf(stderr, p_format, argp);
+	va_end(argp);
+	fprintf(stderr, "\n");
+}
+
+static void printverbose(const char *p_format, ...)
+{
+	va_list argp;
+
+	fprintf(stderr, "[dvdspanky] verbose: ");
+	va_start(argp, p_format);
+	vfprintf(stderr, p_format, argp);
+	va_end(argp);
+	fprintf(stderr, "\n");
+}
+
 /* termination cleanup */
 void cleanup()
 {
@@ -176,6 +215,10 @@ int encode(video_t *p_in, video_t *p_out, int options, const char *p_filters)
 	/* filters */
 	if(options & SPANK_DOUBLEFPS)
 		safecat(" -J doublefps");
+	if(options & SPANK_PPROCESS)
+		safecat(" -J pp=de");
+	if(options & SPANK_NORMALISE)
+		safecat(" -J normalize");
 	if(p_filters[0])
 	{
 		safecat(" -J ");
@@ -371,8 +414,6 @@ int cb_tcprobe(const char *p_string, void *p_options)
 		return 0;
 	}
 
-	/* fprintf(stderr, "[debug] tcp: %s\n", p_string); */
-
 	/* video regex */
 	length = strlen(p_string);
 	numstrings = pcre_exec(gp_tcvideoregex, NULL, p_string, length, 0, 0,
@@ -487,7 +528,7 @@ int cb_transcode(const char *p_string, void *p_options)
 	}
 	else if(p_video->options & SPANK_VERBOSE)
 	{
-		fprintf(stderr, "%s\n", p_buffer);
+		printverbose("%s\n", p_buffer);
 	}
 
 	return 0;
@@ -754,85 +795,49 @@ int main(int argc, char **argv)
 	{
 		switch(opt)
 		{
-			case 'i':
-				strncpy(invid.p_filename, optarg, sizeof(invid.p_filename) - 1);
+			case '1':
+				options |= SPANK_FIRST;
 				break;
-			case 'o':
-				strncpy(outvid.p_filename, optarg, sizeof(outvid.p_filename) - 1);
+			case '2':
+				options |= SPANK_SECOND;
 				break;
-			case 'r':
-				vrate = atoi(optarg);
-				if(vrate < 600 || vrate > 7500)
+			case 'a':
+				aspect = atoi(optarg);
+				if(aspect != 2 && aspect != 3)
 				{
-					fprintf(stderr, "error: vrate not 600 - 7500\n");
+					printerror("aspect not 2 or 3");
 					exit(1);
 				}
 				break;
-			case 'R':
-				arate = atoi(optarg);
-				if(arate < 64 || arate > 448)
-				{
-					fprintf(stderr, "error: arate not 64 - 448\n");
-					exit(1);
-				}
-				break;
-			case 's':
-				calcsize = atoi(optarg);
-				break;
-			case 'c':
-				channels = atoi(optarg);
-				if(!channels || channels > 6)
-				{
-					fprintf(stderr, "error: channels not 1 - 6\n");
-					exit(1);
-				}
-				break;
-			case 'V':
-				vcrop = atoi(optarg);
-				break;
-			case 'H':
-				hcrop = atoi(optarg);
-				break;
 			case 'b':
 				calcborder = 1;
 				break;
 			case 'B':
 				calcclip = 1;
 				break;
-			case 'p':
-				vidmode = 'p';
+			case 'c':
+				channels = atoi(optarg);
+				if(!channels || channels > 6)
+				{
+					printerror("channels not 1-6");
+					exit(1);
+				}
 				break;
-			case 'n':
-				vidmode = 'n';
-				break;
-			case 'N':
-				options |= SPANK_FIXNTSC;
+			case 'f':
+				strncpy(p_filters, optarg, sizeof(p_filters) - 1);
 				break;
 			case 'F':
 				options |= SPANK_NEWFPS;
 				break;
-			case 'a':
-				aspect = atoi(optarg);
-				if(aspect != 2 && aspect != 3)
-				{
-					fprintf(stderr, "error: aspect not 2 or 3\n");
-					exit(1);
-				}
+			case 'h':
+				printf(gp_help, argv[0]);
+				exit(0);
 				break;
-			case 'm':
-				options |= SPANK_FIRST | SPANK_SECOND;
+			case 'H':
+				hcrop = atoi(optarg);
 				break;
-			case 'M':
-				options |= SPANK_MULTIPLEX;
-				break;
-			case '1':
-				options |= SPANK_FIRST;
-				break;
-			case '2':
-				options |= SPANK_SECOND;
-				break;
-			case 'f':
-				strncpy(p_filters, optarg, sizeof(p_filters) - 1);
+			case 'i':
+				strncpy(invid.p_filename, optarg, sizeof(invid.p_filename) - 1);
 				break;
 			case 'I':
 				options |= SPANK_NONICE;
@@ -841,21 +846,63 @@ int main(int argc, char **argv)
 				printf(gp_licence);
 				exit(0);
 				break;
-			case 'h':
-				printf(gp_help, argv[0]);
-				exit(0);
+			case 'm':
+				options |= SPANK_FIRST | SPANK_SECOND;
+				break;
+			case 'M':
+				options |= SPANK_MULTIPLEX;
+				break;
+			case 'n':
+				vidmode = 'n';
+				break;
+			case 'N':
+				options |= SPANK_FIXNTSC;
+				break;
+			case 'p':
+				vidmode = 'p';
+				break;
+			case 'P':
+				options |= SPANK_PREVIEW;
+				break;
+			case 'o':
+				strncpy(outvid.p_filename, optarg, sizeof(outvid.p_filename) - 1);
+				break;
+			case 'O':
+				options |= SPANK_NORMALISE;
+				break;
+			case 'r':
+				vrate = atoi(optarg);
+				if(vrate < 600 || vrate > 7500)
+				{
+					printerror("vrate not 600 - 7500");
+					exit(1);
+				}
+				break;
+			case 'R':
+				arate = atoi(optarg);
+				if(arate < 64 || arate > 448)
+				{
+					printerror("arate not 64 - 488");
+					exit(1);
+				}
+				break;
+			case 's':
+				calcsize = atoi(optarg);
+				break;
+			case 'T':
+				options |= SPANK_PPROCESS;
 				break;
 			case 'v':
 				options |= SPANK_VERBOSE;
 				break;
-			case 'P':
-				options |= SPANK_PREVIEW;
+			case 'V':
+				vcrop = atoi(optarg);
 				break;
 			case '?':
 				exit(1);
 				break;
 			default:
-				fprintf(stderr, "error: unhandled option %c\n", opt);
+				printerror("unhandled option %c", opt);
 				exit(1);
 				break;
 		}
@@ -865,15 +912,37 @@ int main(int argc, char **argv)
 	if(!invid.p_filename[0])
 	{
 		/* TODO: check for existance of file */
-		fprintf(stderr, "error: no input file\n");
+		printerror("no input file");
 		exit(1);
 	}
 
-	/* test output filename */
+	/* create output filename based on input filename if none given */
 	if(!outvid.p_filename[0])
 	{
-		fprintf(stderr, "error: no output file\n");
-		exit(1);
+		pcre *p_regex;
+		int p_vector[10];
+		int found;
+
+		p_regex = pcre_compile("^.*?([^/.]+)(?:\\.|)[^/.]*$", 0, &p_error, &erroffset, NULL);
+		if(!p_regex)
+		{
+			printerror("failed to compile filename regex");
+			exit(1);
+		}
+		found = pcre_exec(p_regex, NULL, invid.p_filename, strlen(invid.p_filename), 0, 0,
+			p_vector, sizeof(p_vector) / sizeof(int));
+		if(found > 0)
+		{
+			strncpy(outvid.p_filename, &invid.p_filename[p_vector[2]], p_vector[3] - p_vector[2]);
+			outvid.p_filename[p_vector[3] - p_vector[2]] = 0;
+		}
+		else
+		{
+			printerror("unable to fathom output filename from input");
+			pcre_free(p_regex);
+			exit(1);
+		}
+		pcre_free(p_regex);
 	}
 
 	/* get source video details */
@@ -910,12 +979,11 @@ int main(int argc, char **argv)
 
 		if(vrate < 600 || vrate > 7500)
 		{
-			fprintf(stderr, "error: calculated video rate %d not 600 - 7500 for size %d KB\n",
-				vrate, calcsize);
-			calcsize = ((631 + arate + arate / 20) / 8 * invid.length) / 1000 * 1000;
-			fprintf(stderr, "error: suggested minimum size setting of %d KB\n", calcsize);
-			calcsize = ((7875 + arate + arate / 20) / 8 * invid.length) / 1000 * 1000;
-			fprintf(stderr, "error: suggested maximum size setting of %d KB\n", calcsize);
+			printerror("calculated video rate %d not 600 - 7500 for size %d KB", vrate, calcsize);
+			printerror("suggested minimum size setting of %d KB",
+				((631 + arate + arate / 20) / 8 * invid.length) / 1000 * 1000);
+			printerror("suggested maximum size setting of %d KB",
+				((7875 + arate + arate / 20) / 8 * invid.length) / 1000 * 1000);
 			exit(1);
 		}
 	}
@@ -1012,7 +1080,7 @@ int main(int argc, char **argv)
 			vcrop = -(invid.width * 10 / 4 * 3 - invid.height * 10) / 40 * 2;
 			if(vcrop > 0)
 			{
-				hcrop = -vcrop;
+				hcrop = (invid.width * 10 - invid.height * 10 / 3 * 4) / 40 * 2;
 				vcrop = 0;
 			}
 		}
@@ -1021,7 +1089,7 @@ int main(int argc, char **argv)
 			vcrop = -(invid.width * 10 / 16 * 9 - invid.height * 10) / 40 * 2;
 			if(vcrop > 0)
 			{
-				hcrop = -vcrop;
+				hcrop = (invid.width * 10 - invid.height * 10 / 9 * 16) / 40 * 2;
 				vcrop = 0;
 			}
 		}
@@ -1034,7 +1102,7 @@ int main(int argc, char **argv)
 			vcrop = -(invid.width * 10 / 4 * 3 - invid.height * 10) / 40 * 2;
 			if(vcrop < 0)
 			{
-				hcrop = -vcrop;
+				hcrop = (invid.width * 10 - invid.height * 10 / 3 * 4) / 40 * 2;
 				vcrop = 0;
 			}
 		}
@@ -1043,7 +1111,7 @@ int main(int argc, char **argv)
 			vcrop = -(invid.width * 10 / 16 * 9 - invid.height * 10) / 40 * 2;
 			if(vcrop < 0)
 			{
-				hcrop = -vcrop;
+				hcrop = (invid.width * 10 - invid.height * 10 / 9 * 16) / 40 * 2;
 				vcrop = 0;
 			}
 		}
@@ -1053,22 +1121,18 @@ int main(int argc, char **argv)
 
 	/* print source and dest video encoding info */
 	printf(
-		"source  %s\n"
-		" video  format=%s  resolution=%dx%d  rate=%d kbps\n"
-		"        aspect=%.4f  frames=%d  fps=%.3f\n"
-		"        time=%d:%02d:%02d\n"
-		" audio  format=%s  rate=%d kbps  hz=%d\n"
-		"        bits=%d  channels=%d\n",
+		" source | %s\n"
+		"  video | %s, %dx%d, %d kbps, %.3f fps\n"
+		"        | %.4f aspect, %d frames, %d:%02d:%02d\n"
+		"  audio | %s, %d kbps, %d Hz, %d bit, %d channel\n",
 		invid.p_filename,
-		invid.p_vcodec, invid.width, invid.height, invid.vrate,
-		invid.aspect, invid.frames, invid.fps,
-		invid.hours, invid.minutes, invid.seconds,
-		invid.p_acodec, invid.arate, invid.hz,
-		invid.abits, invid.achans
+		invid.p_vcodec, invid.width, invid.height, invid.vrate, invid.fps,
+		invid.aspect, invid.frames,	invid.hours, invid.minutes, invid.seconds,
+		invid.p_acodec, invid.arate, invid.hz, invid.abits, invid.achans
 		);
 	if(options & SPANK_MPLAYER)
 		printf(
-		"   misc  mplayer decoder\n"
+		"   info | using mplayer decoder\n"
 		);
 	printf(
 		"\n"
@@ -1076,21 +1140,18 @@ int main(int argc, char **argv)
 
 
 	printf(
-		"  dest  %s.mpg\n"
-		" video  format=%s  resolution=%dx%d  rate=%d kbps\n"
-		"        aspect=%.4f  frames=%d  fps=%.3f\n"
-		"        time=%d:%02d:%02d  vcrop=%d  hcrop=%d\n"
-		" audio  format=%s  rate=%d kbps  hz=%d\n"
-		"        bits=%d  channels=%d\n"
-		"  misc  filesize=~%d, encode=%s\n\n",
+		"   dest | %s.mpg\n"
+		"  video | %s, %dx%d, %d kbps, %.3f fps\n"
+		"        | %.4f aspect, %d frames, %d:%02d:%02d\n"
+		"        | %d:%d crop, %s\n"
+		"  audio | %s, %d kbps, %d Hz, %d bit, %d channel\n"
+		"   file | ~%d KB\n\n",
 		outvid.p_filename,
-		outvid.p_vcodec, outvid.width, outvid.height, outvid.vrate,
-		outvid.aspect, outvid.frames, outvid.fps,
-		outvid.hours, outvid.minutes, outvid.seconds, outvid.vcrop, outvid.hcrop,
-		outvid.p_acodec, outvid.arate, outvid.hz,
-		outvid.abits, outvid.achans,
-		(vrate + vrate / 20 + arate + arate / 20) / 8 * invid.length,
-		(options & SPANK_MULTI) ? "VBR" : "CBR"
+		outvid.p_vcodec, outvid.width, outvid.height, outvid.vrate, outvid.fps,
+		outvid.aspect, outvid.frames, outvid.hours, outvid.minutes, outvid.seconds,
+		outvid.vcrop, outvid.hcrop, (options & SPANK_MULTI) ? "VBR" : "CBR",
+		outvid.p_acodec, outvid.arate, outvid.hz, outvid.abits, outvid.achans,
+		(vrate + vrate / 20 + arate + arate / 20) / 8 * invid.length
 		);
 
 	exit(encode(&invid, &outvid, options, p_filters));
